
from apgl.predictors.AbstractPredictor import AbstractPredictor
from apgl.util import * 
import logging
import numpy

class AbstractSVMPredictor(AbstractPredictor):
    def cvModelSelection(self, X, y, Cs, kernelParams, kernel, folds, errorCosts, errorMethod=Evaluator.binaryError, epsilons=[0.1]):
        """
        A function to perform model selection for the penalty parameter of the SVM
        """
        Parameter.checkString(kernel, ["linear", "polynomial", "gaussian"])
        Parameter.checkInt(folds, 2, X.shape[0])
        
        numIterations = len(Cs)*len(kernelParams)*len(errorCosts)*len(epsilons)
        idx = Sampling.crossValidation(folds, y.shape[0])

        logging.info("Starting " + str(folds) + "-fold SVM model selection with " + str(X.shape[0]) + " examples")
        logging.info("Using " + str(errorMethod) + " for evaluation.")
        iter = 0

        best = {}
        best["error"] = float("inf")

        for C in Cs:
            for kp in kernelParams:
                for errorCost in errorCosts:
                    for epsilon in epsilons: 
                        logging.info("Model selection iteration " + str(iter) + " of " + str(numIterations-1))
                        
                        iter = iter + 1 

                        self.setKernel(kernel, kp)
                        self.setC(C)
                        self.setErrorCost(errorCost)
                        self.setEpsilon(epsilon)
                        logging.info(str(self))

                        error = 0.0
                        m = 0

                        for idxtr, idxts in idx:
                            trainX, testX = X[idxtr], X[idxts]       # build training data
                            trainY, testY = y[idxtr], y[idxts]       # build test data

                            self.learnModel(trainX, trainY)
                            predY = self.predict(testX)
                            error += errorMethod(testY, predY)          # compute the prediction error
                            m += 1

                        error = error/len(idx)

                        if error < best["error"]:
                            best["error"] = error
                            best["C"] = C
                            best["kp"] = kp
                            best["epsilon"] = epsilon
                            best["errorCost"] = errorCost
                            logging.info("Found new best error: " + str(error))

        return best

    def setC(self, C):
        Util.abstract()

    def setKernel(self, kernel, kernelParam):
        Util.abstract()

    def setErrorCost(self):
        Util.abstract()

